/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef COLORS_H
#define COLORS_H

#include <QtCore>
#include <QColor>





class _colorCount
{
public:
 QRgb _color;
 int _count;
 bool operator < (const _colorCount &cc) const {return _count<cc._count;}
};


class _dmc
{
  //см. http://faqs.org.ru/progr/graph/graph.htm раздел: 3. 'Близость' ('схожесть') цветов.



  //----------------------------------------------------------------------------
public:

  class YUV{
    const float y,u,v;
  public:
    YUV(const QColor& cl):y(cl.red()*0.299+cl.green()*0.587+cl.blue()*0.114),
                          u(cl.red()*(-0.16874)+cl.green()*(-0.33126)+cl.blue()*0.5+128),
                          v(cl.red()*0.5+cl.green()*(-0.41869)+cl.blue()*(-0.08131)+128){}
    float distance(const YUV& x)const{return sqrt(pow(y-x.y,2)*4+pow(u-x.u,2)+pow(v-x.v,2));}
  };

  class RGB{
    const float gamma;
    const float r,g,b;
  public:
    RGB(const QColor& cl):gamma(1.4),
      r(pow(cl.red(),gamma)*0.3),
      g(pow(cl.green(),gamma)*0.59),
      b(pow(cl.blue(),gamma)*0.11)
      //r(cl.red()),
      //g(cl.green()),
      //b(cl.blue())
    {}
    float distance(const RGB& x)const{return sqrt(pow(r-x.r,2)+pow(g-x.g,2)+pow(b-x.b,2));}
  };


  struct colorDMC
  {
   QRgb    _color;
   QString _number;
   QString _description;
  };
  QList<colorDMC>color;//Цвета палитры DMC

  _dmc();
  ~_dmc(){color.clear();}
  QRgb find(const QColor& cl,int pos=0)const;
  static QRgb blank(){return 0x00000000;}//Прозрачный чёрный
  static float distance(QRgb c1,QRgb c2){return sqrt(pow(qRed(c1)-qRed(c2),2)+pow(qGreen(c1)-qGreen(c2),2)+pow(qBlue(c1)-qBlue(c2),2));}
private:
  void load(const QString &path);
};

#endif // COLORS_H
