/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "comboboxdelegate.h"
#include "info.h"


const char NO_ICON=-1;


comboBoxDelegate::comboBoxDelegate(QObject *parent):QItemDelegate(parent),
  _iconPicList(),
  _iconColList(),
  dmc()
{}


comboBoxDelegate::~comboBoxDelegate()
{
  _iconPicList.clear();
  _iconColList.clear();
  dmc.~_dmc();
}


QWidget *comboBoxDelegate::createEditor
(QWidget *parent, const QStyleOptionViewItem &,const QModelIndex &index) const
{
  QComboBox *editor = new QComboBox(parent);

  switch(index.column()){
    case 0:
      if(index.row()<_iconColList.size()){
        //Убираем те иконки, которые встречаются в других строках таблицы и заполняем таблицу
                        QString currtext=index.model()->data(index,Qt::DisplayRole).toString();
                        for(int i=0;i<_iconColList.size();i++){
                          bool stop=false;
                          QString text=dmc.color.at(i)._number+" "+dmc.color.at(i)._description;
                          if(currtext==text){
                            editor->insertItem(0,_iconColList.at(i),text);
                            editor->setItemData(0,uint(dmc.color.at(i)._color));
                          }
                          for(int k=0;k<index.model()->rowCount();k++){
                            if(index.model()->data(index.model()->index(k,0),Qt::DisplayRole).toString()==text){
                              stop=true;
                              break;
                            }
                          }
                          if(!stop){
                            editor->addItem(_iconColList.at(i),text);
                            editor->setItemData(editor->count()-1,uint(dmc.color.at(i)._color));
                          }
                        }
        //------------------------------------------------------------------------------
        editor->setCurrentIndex(0);
        break;
      }

    case 1:
      if(index.row()<_iconPicList.size()){
//Убираем те иконки, которые встречаются в других строках таблицы
              QList<QIcon>iconList;
              QList<int>id;
              for(int i=0;i<index.model()->rowCount();i++){
                int num=index.model()->data(index.model()->index(i,1),Qt::UserRole).toInt();
                id.append(num);
              }

              for(int i=0;i<_iconPicList.size();i++){
                bool stop=false;
                for(int k=0;k<id.size();k++){
                  if(id.at(k)==i){
                    stop=true;
                    break;
                  }
                }
                if(!stop)iconList.append(_iconPicList.at(i));
              }
//------------------------------------------------------------------------------

        int val=index.model()->data(index,Qt::UserRole).toInt();
        if(val!=NO_ICON)iconList.prepend(_iconPicList.at(val));
        for(int i=0;i<iconList.size();i++)editor->addItem(iconList.at(i),"");
        editor->setCurrentIndex(0);
        break;
      }

    default:{
      delete editor;
      return 0;
    }
  }
  editor->installEventFilter(const_cast<comboBoxDelegate*>(this));
  return editor;
}


void comboBoxDelegate::setModelData
(QWidget *editor, QAbstractItemModel *model,const QModelIndex &index) const
{
  QComboBox *comboBox=static_cast<QComboBox*>(editor);
  int ci=comboBox->currentIndex();

  if(index.column()==0){
      QString text=comboBox->itemText(ci);
      QRgb cl(comboBox->itemData(ci).toUInt());

      QMap<int,QVariant> map;
      map.insert(Qt::DecorationRole,QColor(cl));
      map.insert(Qt::UserRole,cl);
      map.insert(Qt::DisplayRole,text);
      model->setItemData(index,map);
  }

  if(index.column()==1){
      QIcon icon=comboBox->itemIcon(ci);
//Находим идентификтор иконки
      int num=NO_ICON;
      for(int i=0;i<_iconPicList.size();i++)
        if(_iconPicList.at(i).cacheKey()==icon.cacheKey()){
          num=i;
          break;
        }
//------------------------------------------------------------------------------
      QMap<int,QVariant> map;
      map.insert(Qt::DecorationRole,icon);
      map.insert(Qt::UserRole,num);
      model->setItemData(index,map);
  }
}


void comboBoxDelegate::setSize(int x,int y){
  setPicIcons(x,y);
  setColIcons(x,y);
}


void comboBoxDelegate::setPicIcons(int x,int y){
  _iconPicList.clear();
  QDir clpicDir(Info::dataPath()+"/system/clpics");
  QStringList filter;
  filter.append("clpic*.bmp");
  int clpicCount=clpicDir.entryList(filter,QDir::Files).count();

  for(int i=0;i<clpicCount;i++){
    QImage image(Info::dataPath()+"/system/clpics/clpic"+QString::number(i)+".bmp");
    image=image.scaled(x,y);
    image.setAlphaChannel(image.createMaskFromColor(0xFFFFFFFF,Qt::MaskOutColor));
    QIcon icon(QPixmap::fromImage(image));
    _iconPicList.append(icon);
  }
}


void comboBoxDelegate::setColIcons(int x,int y){
   _iconColList.clear();
   for(int i=0;i<dmc.color.size();i++){
     QImage image(x,y,QImage::Format_RGB32);
     image.fill(dmc.color.at(i)._color);
     QIcon icon(QPixmap::fromImage(image));
     _iconColList.append(icon);
   }
}

