/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "history.h"
#include "info.h"


SpriteItem::SpriteItem(QList<Sprite*>* sptr,QObject *parent):HistoryItem(parent),
  spritePtr(sptr),leaf()
{}

bool SpriteItem::contains(QRgb cl)
{
  for(int k=0;k<leaf.size();k++)
    if(leaf.at(k).color.rgba()==cl)return true;
  return false;
}

void SpriteItem::execute(){
  for(int i=0;i<leaf.size();i++)
    //iid=-1 используется для ведения истории изменения цвета линий (используется в ColorItem)
    if(leaf.at(i).iid!=-1){
      Data h(leaf.at(i));
      Data h2(h.index,(*spritePtr).at(h.index)->iconID(),(*spritePtr).at(h.index)->color(),(*spritePtr).at(h.index)->stitch());
      leaf.replace(i,h2);
      if(h.color.isValid())(*spritePtr)[h.index]->setColor(h.color);
      (*spritePtr)[h.index]->setIconID(h.iid);
      (*spritePtr)[h.index]->setStitch(h.stitch);
    }else break;
}


//------------------------------------------------------------------------------


IconItem::IconItem(int r,const QColor& cl,QList<Sprite*>* sptr,QObject *parent):SpriteItem(sptr,parent),row(r){
  for(int i=0;i<(*spritePtr).size();i++)
    if((*spritePtr).at(i)->color()==cl)
      append(i,(*spritePtr).at(i)->iconID(),QColor());
}

void IconItem::execute(){
  emit changed(row,leaf.at(0).iid);
  SpriteItem::execute();
}


//------------------------------------------------------------------------------


ColorItem::ColorItem(int r,const QColor& cl,QList<Sprite*>* sptr,QList<Line*>* lptr,QObject *parent):SpriteItem(sptr,parent),linePtr(lptr),row(r){
  for(int i=0;i<(*spritePtr).size();i++)
    if((*spritePtr).at(i)->color()==cl)
      append(i,(*spritePtr).at(i)->iconID(),cl);
  for(int i=0;i<(*linePtr).size();i++)
    if((*linePtr).at(i)->color()==cl)
      append(i,-1,cl);
}

void ColorItem::execute(){
  emit changed(row,leaf.at(0).color);
  SpriteItem::execute();

  for(int i=0;i<size();i++)
  if(leaf.at(i).iid==-1){
    Data h(leaf.at(i));
    Data h2(h.index,-1,(*linePtr).at(h.index)->color());
    leaf.replace(i,h2);
    if(h.color.isValid())(*linePtr)[h.index]->setColor(h.color);
  }
}


//------------------------------------------------------------------------------


LineItem::LineItem(const Line& ln,QList<Line*>* lptr,QObject *parent):HistoryItem(parent),
    line(ln),linePtr(lptr)
{}

bool LineItem::contains(QRgb cl)
{
  return cl==line.color().rgba();
}

void LineItem::execute(){
    //Просматриваем весь массив line, если находим соответствие hn.line(), удаляем её из line;
    //если нет, то добавляем hn.line() в line
               int id=(*linePtr).size();
               for(int i=0;i<id;i++)
                 if(line==*(*linePtr).at(i)){
                   id=i;
                   break;
                 }
               if(id<(*linePtr).size())delete (*linePtr).takeAt(id);
               else{
                 try{
                   (*linePtr).append(new Line(line));
                 }catch(const std::bad_alloc&){
                   //Exception handle
                   Info::messageSplash(Info::msgOutOfMemory);
                 }
               }
}


//------------------------------------------------------------------------------


History::History(QObject *parent):QObject(parent),
  item(),marker(-1),undo(false),redo(false)
{}

void History::append(HistoryItem* hi){
  removeLast();
  item.append(hi);
  marker=item.size();
}

void History::append(int i,int iid,const QColor& cl,Sprite::Stitch st)
{
  SpriteItem* si=dynamic_cast<SpriteItem*>(item.last());
  if(si!=0)si->append(i,iid,cl,st);
}

void History::execute(){
  item.at(marker)->execute();
}

void History::clear(){
  marker=-1;
  undo=false;
  redo=false;
  while(!item.isEmpty())
    delete item.takeFirst();
  item.clear();
  emit redoEnable(false);
  emit undoEnable(false);
}

void History::rwd(){
   undo=true;
   if(!redo)marker--;else redo=false;
   execute();
   emit redoEnable(true);
   if(marker==0)emit undoEnable(false);
}

void History::fwd(){
   redo=true;
   if(!undo)marker++;else undo=false;
   execute();
   emit undoEnable(true);
   if(marker==item.size()-1)emit redoEnable(false);
}

void History::remove(QRgb rgbcl){
  //При удалении цвета удаляет упоминания о нём из истории

  //rcount cчитает сколько элементов надо удалить перед маркером
  //Это нужно чтобы маркер указывал на тот же элемент после удаления некоторых элементов массива
  int rcount=0;
  QList<int>del;//Номера элементов помеченных к удалению
  for(int i=0;i<item.size();i++)
    if(item.at(i)->contains(rgbcl)){
      del.append(i);
      if(i<marker)rcount++;
    }
  marker-=rcount;

  //Удаляем элементы массива
  for(int i=0;i<del.size();i++)
    delete item.takeAt(del.at(i)-i);//будь внимателен, (-i) здесь очень важно

  if(marker<=0)emit undoEnable(false);
  if(marker>item.size()-1)emit redoEnable(false);
  if(item.size()==0){
    emit undoEnable(false);
    emit redoEnable(false);
  }
}

void History::removeLast(){
  if(item.size()!=0){
    int start=item.size();
    if(undo)start=marker;
    if(redo)start=marker+1;
    int size=item.size();
    for(int k=start;k<size;k++)delete item.takeLast();
  }
  undo=false;
  redo=false;
  emit undoEnable(true);
  emit redoEnable(false);
}

