/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef HISTORY_H
#define HISTORY_H

#include "line.h"
#include "sprite.h"
#include<QColor>
#include<QList>
#include<QObject>

#include<typeinfo>




class HistoryItem:public QObject
{
public:
  HistoryItem(QObject *parent=0):QObject(parent){}
  virtual ~HistoryItem(){}
  virtual void execute()=0;
  virtual bool contains(QRgb)=0;
};


class LineItem:public HistoryItem{
  friend class History;
  Line          line;//История линии
  QList<Line*>* linePtr;//Ссылка на список линий из View
  void execute();
  bool contains(QRgb cl);
public:
  LineItem(const Line& ln,QList<Line*>* lptr,QObject *parent=0);
};


class SpriteItem:public HistoryItem{
  friend class History;
  bool contains(QRgb cl);
protected:

  class Data{
  public:
    int            index;
    int            iid;
    QColor         color;
    Sprite::Stitch stitch;
    Data(int i,int id,const QColor& cl,Sprite::Stitch st=Sprite::Cross):index(i),iid(id),color(cl),stitch(st){}
    Data(const Data& h):index(h.index),iid(h.iid),color(h.color),stitch(h.stitch){}
  };

  QList<Sprite*>* spritePtr;//Ссылка на список спрайтов из View
  QList<Data>     leaf;//Массив для сохранения истории
  void append(int index,int iid,const QColor& color,Sprite::Stitch st=Sprite::Cross){leaf.append(Data(index,iid,color,st));}
  void execute();
public:
  SpriteItem(QList<Sprite*>* sptr,QObject *parent=0);
  ~SpriteItem(){leaf.clear();}
  int size()const{return leaf.size();}
};


class IconItem:public SpriteItem{
  Q_OBJECT
  int row;
  void execute();
signals:
  void changed(int,int);
public:
  IconItem(int r,const QColor& cl,QList<Sprite*>* sptr,QObject *parent=0);
};


class ColorItem:public SpriteItem{
  Q_OBJECT
  QList<Line*>* linePtr;//Ссылка на список линий из View
  int row;
  void execute();
signals:
  void changed(int,QColor);
public:
  ColorItem(int r,const QColor& cl,QList<Sprite*>* sptr,QList<Line*>* lptr,QObject *parent=0);
};


class History:public QObject{
  Q_OBJECT
  QList<HistoryItem*> item;
  int                 marker;
  bool                undo;
  bool                redo;
  void removeLast();
  void execute();
signals:
  void undoEnable(bool);
  void redoEnable(bool);
public:
  History(QObject *parent=0);
  void append(HistoryItem* hi);
  void append(int i,int iid,const QColor& cl,Sprite::Stitch st=Sprite::Cross);
  void clear();
  void remove(QRgb rgbcl);
public slots:
  void rwd();
  void fwd();
};


#endif // HISTORY_H
