/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "imagefilter.h"


_imageFilter::_imageFilter(int edge)
{
 setEdge(edge);
}


void _imageFilter::setEdge(int edge)
{
 _edge=edge;
 setWeight();
}


QImage _imageFilter::filter(QImage image,int mode,QProgressDialog *pb)
{

 setWeight(mode);

 for(int i=_edge;i<image.width()-_edge;i++){
   for(int j=_edge;j<image.height()-_edge;j++){

     QList<QRgb>pixel;
     for(int ik=-_edge;ik<_edge+1;ik++)
       for(int jk=-_edge;jk<_edge+1;jk++)
         pixel.append(image.pixel(i+ik,j+jk));


     int clR=0,clG=0,clB=0;
     for(int k=0;k<_weight.size();k++){
         clR+=_weight.at(k)*qRed(pixel.at(k));
         clG+=_weight.at(k)*qGreen(pixel.at(k));
         clB+=_weight.at(k)*qBlue(pixel.at(k));
     }


     clR/=_div;
     clG/=_div;
     clB/=_div;

     if(clR>255)clR=255;
     if(clG>255)clG=255;
     if(clB>255)clB=255;
     if(clR<0)clR=0;
     if(clG<0)clG=0;
     if(clB<0)clB=0;

     image.setPixel(i,j,qRgb(clR,clG,clB));

      if(pb!=0)
        if(pb->wasCanceled())return QImage();

    }

   if(pb!=0)pb->setValue(pb->value()+1);

 }
 return image;
}


QImage _imageFilter::medium(QImage image,QProgressDialog *pb)
{
  for(int i=0;i<image.width();i++){
    for(int j=0;j<image.height();j++){
      int i1=image.pixelIndex(i,j);
      if(j<image.height()-3){
        int i2=image.pixelIndex(i,j+2);
        if(i1==i2)image.setPixel(i,j+1,i1);
      }
      if(i<image.width()-3){
        int i2=image.pixelIndex(i+2,j);
        if(i1==i2)image.setPixel(i+1,j,i1);
      }

      if(pb!=0)
        if(pb->wasCanceled())return QImage();
    }
    if(pb!=0)pb->setValue(pb->value()+1);
  }
 return image;
}

QImage _imageFilter::median(QImage image,QVector<QRgb>colorTable,QProgressDialog *pb)
{
  for(int i=0;i<image.width();i++){
    QList<int>index;
    QList<int>mid;
    mid.append(image.pixelIndex(i,0));
    mid.append(image.pixelIndex(i,0));
    for(int j=0;j<image.height();j++){
      if(j<image.height()-1)mid.append(image.pixelIndex(i,j+1));
      else mid.append(image.pixelIndex(i,j));
      if(j!=0)mid.removeFirst();
      index.append(MIDfilter(colorTable,mid));
    }

    for(int j=0;j<image.height();j++)image.setPixel(i,j,index.at(j));
    if(pb!=0){
      pb->setValue(pb->value()+1);
      if(pb->wasCanceled())return QImage();
    }
  }

  for(int i=0;i<image.height();i++){
    QList<int>index;
    QList<int>mid;
    mid.append(image.pixelIndex(0,i));
    mid.append(image.pixelIndex(0,i));
    for(int j=0;j<image.width();j++){
      if(j<image.width()-1)mid.append(image.pixelIndex(j+1,i));
      else mid.append(image.pixelIndex(j,i));
      if(j!=0)mid.removeFirst();
      index.append(MIDfilter(colorTable,mid));
    }

    for(int j=0;j<image.width();j++)image.setPixel(j,i,index.at(j));
    if(pb!=0){
      pb->setValue(pb->value()+1);
      if(pb->wasCanceled())return QImage();
    }
  }

  return image;
}


int _imageFilter::MIDfilter(const QVector<QRgb> &colorTable,QList<int> midval){
 for(int i=0;i<midval.size();i++)
   for(int j=0;j<midval.size()-1-i;j++)
     if(colorTable.at(midval.at(j))>colorTable.at(midval.at(j+1)))
       midval.swap(j,j+1);
 return midval.at(int(midval.size()/2));
}


void _imageFilter::setDiv()
{
 _div=0;
 for(int i=0;i<_weight.size();i++)_div+=_weight.at(i);
 if(!_div)_div=1;
}

void _imageFilter::setWeight(int mode){
 _weight.clear();
 switch(mode){
   //Нет фильтра
   case 0:{
             for(int i=-_edge;i<_edge+1;i++)
               for(int j=-_edge;j<_edge+1;j++)
                 if(!i && !j)_weight.append(1);//Эквивалент (i==0)&(j==0)
                 else _weight.append(0);
          };break;
   //Блюр
   case 1:{
             for(int i=-_edge;i<_edge+1;i++)
               for(int j=-_edge;j<_edge+1;j++)_weight.append(1);
          };break;
   //Тиснение (лёгкое)
   case 2:{
             for(int i=-_edge;i<_edge+1;i++)
               for(int j=-_edge;j<_edge+1;j++){
                 if(i==j){
                   if(i<0)_weight.append(-1);else _weight.append(1);
                 }else _weight.append(0);
               }
          };break;
   //---Резкость
   case 3:{
             float mul=1;
             mul/=4*_edge;
             for(int i=-_edge;i<_edge+1;i++)
               for(int j=-_edge;j<_edge+1;j++){
                 if(!i && !j)_weight.append(2);//_weight.append(4*_edge*mul+1);
                 else{
                   if(!i || !j)_weight.append(-mul);else _weight.append(0);
                 }
               }
          };break;
   //Контуры
   case 4:{
             float mul=1;
             mul/=4*_edge;
             for(int i=-_edge;i<_edge+1;i++)
               for(int j=-_edge;j<_edge+1;j++){
                 if(!i && !j)_weight.append(4*_edge*mul);
                 else{
                   if(!i || !j)_weight.append(0);else _weight.append(-mul);
                 }
               }
          };break;
 }
 setDiv();
}



float _imageFilter::scaleFit(const QSize &size,int div){
//Эта ф-ция нужна для масштабирования, т.к. QImage не работает с большими картинками
//Ф-ция масштабирует большие картинки, размер которых слишком велик, к максимально возможному размеру
  float scale=1;

  qreal pixelCount=qreal(size.height())*qreal(size.width());
  if(div==0)div=128;
  float maxPixelCount=INT_MAX/div;//По идее INT_MAX/4 т.к. QImage::Format_ARGB32_Premultiplied занимает 32 бита, т.е. 4 байта
                                 //Эмпирическим путём взято 128)))
                                 //См. http://stackoverflow.com/questions/7080052/qimage-qpixmap-size-limitations

  if(pixelCount>=maxPixelCount){
    int maxwh=(size.height()>size.width())?size.height():size.width();
    scale=pow(maxPixelCount,0.5);
    scale/=maxwh;
  }

  return scale;
}

