/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "info.h"


QString* Info::_dataPath=NULL;
QWidget* Info::_parent=NULL;

QStringList Info::messageString=QStringList()
                                <<trUtf8("Can't find printer!")
                                <<trUtf8("Printer error!")
                                <<trUtf8("Print area error!")
                                <<trUtf8("Image wasn't saved!")
                                <<trUtf8("Scheme wasn't saved!")
                                <<trUtf8("Insufficient memory!")
                                <<trUtf8("Please wait!");



QString Info::colorSign(int i)
{
  const char letter[]="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  QString result="";
  uint rem=i%10;
  uint num=uint((i-rem)/10);
  if(num<sizeof(letter))result=letter[num]+QString::number(rem);
  return result;
}


void Info::aboutQt()
{
  QMessageBox::aboutQt(_parent,QApplication::applicationName()+trUtf8(" - About Qt"));
}


void Info::websiteVisit()
{
  QDesktopServices::openUrl(QUrl("https://sites.google.com/site/crostiapp",QUrl::TolerantMode));
}


void Info::about()
{
  QFileInfo fi(QCoreApplication::applicationFilePath());
  QDateTime created = fi.created();
  QDateTime changed = fi.lastModified();

  QString info="<br>"+trUtf8("Date:")+"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+created.toString("dd.MM.yyyy - hh:mm")
               +"<br><br><br><br>"+trUtf8("Changes:")+"&nbsp;&nbsp;"+changed.toString("dd.MM.yyyy - hh:mm");

  QString text=trUtf8("Application ")+QApplication::applicationName()+"<br><br> "+trUtf8("Version:")+"&nbsp;&nbsp;"
               +QApplication::applicationVersion()+"<br>"+info
      +"<br><br><br><br><br><br><br> "+trUtf8("Author:")+" &copy; "+trUtf8("Levin Sergey")
      +", 2010 - 2012<br> e-mail: <a href=\"mailto:liteswamp@gmail.com\"><b>liteswamp@gmail.com</b></a><br><br>"
      +trUtf8(" Visit website:")+"<br>"
      +" <a href=\"https://sites.google.com/site/crostiapp\">https://sites.google.com/site/crostiapp</a><br><br>"
      +trUtf8(" Donate (BTC):")+"<br>"
      +" <a href=\"http://blockexplorer.com/address/1LGrb923YCskPwZtB9wDnv9SuDaF3p6F6t\">1LGrb923YCskPwZtB9wDnv9SuDaF3p6F6t</a><br><br><br><br>";


  QMessageBox::about(_parent,QApplication::applicationName()+trUtf8(" - About"),text);
}


void Info::messageSplash(Message msgid,int delay)
{
  if((msgid>=0)&&(msgid<messageString.size()))
    messageSplash(messageString.at(msgid),delay);
}


void Info::messageSplash(QString msgstr,int delay)
{
 const int num=5;
 static int dx=0;
 static QList<QLabel*>labelList;

 if(labelList.size()==num){
   QLabel *lb=labelList.takeFirst();
   lb->close();
   lb->~QLabel();
 }
 if(dx==num)dx=0;


 QLabel *lb=new QLabel(msgstr);

 QSettings rsettings(QApplication::organizationName(),QApplication::applicationName());
 QFont font=rsettings.value("font/mainfont").value<QFont>();
 lb->setFont(font);

 lb->setAlignment(Qt::AlignHCenter);
 lb->setWindowFlags(Qt::ToolTip);
 lb->setStyleSheet("QLabel{border: 2px solid gray;padding: 0 8px;background: yellow;selection-background-color: darkgray;}");

 QFontMetrics fm(font);
 int pixelsWide=fm.width(msgstr)+30;
 int pixelsHigh=fm.height()+5;

 int x=0,y=0;
 if(_parent!=NULL){
   x=int(_parent->x()+(_parent->width()-pixelsWide)/2);
   y=int(_parent->y()+(_parent->height()-pixelsHigh)/2);
 }else{
   QRect grect=QApplication::desktop()->availableGeometry();
   x=int(grect.x()+(grect.width()-pixelsWide)/2);
   y=int(grect.y()+(grect.height()-pixelsHigh)/2);
 }

 lb->setMinimumWidth(pixelsWide);
 lb->setGeometry(x,y-dx*(pixelsHigh+5),pixelsWide,pixelsHigh);

 lb->show();
 QTimer::singleShot(delay,lb,SLOT(close()));
 labelList.append(lb);


 dx++;
}


QString Info::imageFilter(const QList<QByteArray>& baList){
  QString filter=trUtf8("Pictures (*.");
  foreach(QByteArray ba,baList)filter+=QString(ba)+" *.";
  filter.chop(3);
  filter+=")";
  return filter;
}
