/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <QtCore>

#ifdef Q_OS_WIN32
  #ifdef QT_STATIC
     #include <QtPlugin>
     Q_IMPORT_PLUGIN(qjpeg)
     Q_IMPORT_PLUGIN(qgif)
     Q_IMPORT_PLUGIN(qmng)
     Q_IMPORT_PLUGIN(qico)
     Q_IMPORT_PLUGIN(qsvg)
     Q_IMPORT_PLUGIN(qtiff)
  #endif
#endif

#include "info.h"
#include "mainwindow.h"




int main(int argc, char *argv[]){

 QApplication::setGraphicsSystem("raster");
 QApplication app(argc, argv);
 app.setApplicationName(APP_NAME);
 app.setOrganizationName(ORGANIZATION_NAME);
 app.setApplicationVersion(APP_VERSION);

 QString dataPath(DATA_PATH);
 if(dataPath=="")//DATA_PATH!="" на Unix системах
   dataPath=QCoreApplication::applicationDirPath();
 Info::setDataPath(dataPath);


 qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));

 QStringList keyList;
 for(int i=1;i<argc;i++)keyList.append(argv[i]);


 QSettings rsettings(app.organizationName(),app.applicationName());
 QString language="en";
 if(rsettings.contains("language"))
   language=rsettings.value("language").value<QString>();


 int index=keyList.indexOf(QRegExp("-l_*",Qt::CaseSensitive,QRegExp::Wildcard));
 if(index!=-1){
   language=keyList[index].remove(0,3);
   keyList.removeAt(index);
 }


 QTranslator translator_qt, translator_app;
 QString translatorFileQt="qt_"+language;
 QString translatorFileApp="crosti_"+language;

 if(language!="en"){
   //---Загрузка переводчика для внутренних классов qt
   if(translator_qt.load(translatorFileQt,Info::dataPath()+"/system/translations"))
     app.installTranslator(&translator_qt);

   //---Загрузка переводчика программы
   if(translator_app.load(translatorFileApp,Info::dataPath()+"/system/translations"))
     app.installTranslator(&translator_app);
 }



 QFont font;
 if(rsettings.contains("font/mainfont"))
   font=rsettings.value("font/mainfont").value<QFont>();
 QApplication::setFont(font);


 QString fileName="";
 for(int i=0;i<keyList.size();i++){
   QFileInfo fi(keyList.at(i));
   QString suff=fi.suffix().toLower();
   if(fi.exists())
     if((QImageReader::supportedImageFormats().contains(suff.toAscii()))||(suff=="cst")){
       fileName=fi.absoluteFilePath();
       break;
     }
 }

 new MainWindow(fileName);

 return app.exec();
}

