/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H



#include "ui_fmmain.h"
#include "ui_fmprogramsettings.h"

#include "view.h"
#include "theme.h"
#include "wizard.h"
#include "colors.h"
#include "previewlabel.h"
#include "comboboxdelegate.h"


#include <QtGui>



class MainWindow : public QMainWindow
{
Q_OBJECT
 public:
   MainWindow(QString fileName="",QWidget *parent=0);

 private slots:
   void makeScheme(_schemeData* schemedata);
   void updateTab(int index);
   void updateTracking();
   void setColor(const QModelIndex &index);
   void schemeModify(QStandardItem *item);
   void openDialog();
   void saveDialog();
   void saveImageDialog();
   void saveColorTableDialog();
   void saveCrostiDialog();
   void openFile(QAction *act);
   void openTab(QAction *act);
   void openImage(QString fileName);
   void openScheme(QString fileDir);
   void setMainFont();
   void setTheme(QString name);
   void setStyle(int id);
   void setUnits(int id);
   void viewProgramSettings();
   void setLanguage(QString lang){_language=lang;}
   void printImage(QPrinter *printer);
   void printScheme(QPrinter *printer);
   void printCrosti(QPrinter *printer);
   void printColorTable(QPrinter *printer);
   void printDialog();
   void setScaleImage(int factor);
   void setScaleScheme(int factor);
   void setScaleColorTable(int factor);
   void setScaleCrosti(int factor);
   void adjustScrollBar(QScrollBar *scrollBar,double factor);
   void scaleResetBtnPress();
   void scaleResetBtn1Press();
   void scaleResetBtn2Press();
   void scaleResetBtn3Press();
   void schemeBtnPress();
   void schemeCreate();
   void colorBtnPress();
   void backColorBtnPress();
   void previewBtnPress();
   void crostiBtnPress();
   void imageResetBtnPress();
   void colorTableCreate(bool update=false);
   void priceBtnPress();
   void crostiClearBtnPress();
   void uncheckRbx();
   void wizardDialog();
   void wizardAccepted();
   void preview();
   void addColorBtnPress();
   void removeColorBtnPress();
   void colorUpdate(int,QColor);
   void iconUpdate(int,int);
   QDialog::DialogCode saveSchemeDialog();
   int setSquareSBtoolTip();
   void squareChanged(int square);
   void setLine(bool b);
   void setLine(int=0);
   void crostiSizeShow(int val);
   void schemeSizeShow();
   void imageSizeShow(QImage img=QImage());
   void imageRotateLeft();
   void imageRotateRight();
   void imageRotate();
   QImage rotate(QImage image,int angle)const;


 protected:
   void closeEvent(QCloseEvent *e);


 private:
   Ui::fmmain ui;
   Ui::fmProgramSettings uiprogramsettings;


   View *view;
   crostiWizard *wizard;
   previewLabel *lbPreview;
   comboBoxDelegate *delegate;
   _dmc dmc;

   QStandardItemModel *paletteModel;
   QWidget *programsettingsWindow;
   QImage _image,//рабочая картинка
          _imageEtalon;//эталон картинки


   QList<_colorCount>colorCount; //Встречаемость цветов в рисунке
   QList<QRgb>colorTable;        //Таблица цветов
   QList<int>id;                 //нужно для идентификации пиктограммы цвета (иконки) в таблице
   QList<QImage>imageList;

   QMenu *fileMenu,
         *helpMenu,
         *viewMenu;
   QAction *aboutAct,
           *aboutQtAct,
           *viewsettingAct,
           *openAct,
           *saveAct,
           *printAct,
           *undoAct,
           *redoAct,
           *dwCrostiAct,
           *dwImageAct,
           *dwPaletteAct,
           *dwToolAct,
           *wizardAct,
           *previewAct,
           *websiteAct,
           *exitAct;
   QActionGroup *openActionGroup,
                *windowActionGroup;

   QLabel *sizeLabel,
          *infoLabel;


   QGraphicsScene *crostiScene,
                  *imageScene,
                  *colorScene;


   int _style,
       _units,//система измерений
       _clpicCount,
       _angle,
       imgH,
       imgW;
   QFont _mainFont,
         _mainFontHalf,
         _mainFontBold,
         _mainFontHalfBold;
   QPalette _palette,
            _paletteBtnPressed,
            _paletteBtnRised;
   QColor _colorStrBtnPressed,
          _colorStrBtnRised,
          _bkColor;
   QString _schemeName,
           _imageName,
           _crostiName,
           _language,
           _schemePath,//Хранит путь к последней открытой или сохраненной схеме
           _picturePath;//Хранит путь к последнему открытому или сохраненному рисунку
   bool _isSchemeCreated,
        _isSchemeUpdated,
        _isConverted,
        _isColorTableUpdated;//Переменная нужна, в основном, для обновления количества каждого
                             //цвета ColorTable на закладке Палитра


   void resetCrosti();
   void setImageList(int squarePix,int dl);
   void shrinkImageList();
   void setParameters(int squarePix,int linePix,int dl);
   void schemeFromFile(QString fileName);
   void spriteFromImage(QImage image);
   void spriteFromView(int square);
   void createActions();
   void createMenus();
   void setBackground(const QColor &cl);
   void removeColor(int row,QRgb rgbcl);
   void addColor(int row,QRgb rgbcl);
   void clearColor(int row);
   void setColor(int row);
   void showImage(QImage img);
   void saveRegistry(QFileInfo fi);
   void setClPicCount();
   bool isMemoryAvailable(qreal msize=0);
   float random(int Max);
   float lightness(QColor color);//Вычисление компоненты Lightness формата HSL
   int analyzeSuffix(QString ext);
   void checkSuffix(QString *fileName,QString nameFilter);
   int colorID(const QColor& cl)const;
   _schemeData* getSchemeData();
   QPixmap renderPixmap(QGraphicsScene *scene);
};

#endif
