/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "schemedata.h"
#include "progressdialog.h"
#include "imagefilter.h"


const char NO_RESULT=-1;



int random(int Max)
{
 static bool firstTime = true;
 if(firstTime){
            firstTime = false;
            QTime midnight(0, 0, 0);
            srand(midnight.secsTo(QTime::currentTime()));
 }
 return rand()%Max;
}

QRgb noise(QRgb cl,int delta=5){
  int d=int(random(delta)-delta/2);
  uchar r=qRed(cl)+d;
  uchar g=qGreen(cl)+d;
  uchar b=qBlue(cl)+d;
  return qRgb(r,g,b);
}



int _schemeData::selectDistantColor(const QList<QRgb> &pickedColor,const QList<QRgb> &unpickedColor) const
{
  float d=0;
  int result=NO_RESULT;
  for(int k=0;k<pickedColor.size();k++){
    QRgb cl=pickedColor.at(k);
    for(int i=0;i<unpickedColor.size();i++){
      QRgb uc=unpickedColor.at(i);
      float td=sqrt(pow(qRed(cl)-qRed(uc),2)+pow(qGreen(cl)-qGreen(uc),2)+pow(qBlue(cl)-qBlue(uc),2));
      if(d<td){
        d=td;
        result=i;
      }
    }
  }
  return result;
}




bool _schemeData::convert(const _dmc &dmc,const int clSize,int mode)
{
 int edge=1;
 uint max=5;
 const uchar iteration=30;//см. [1]
 max+=iteration;

 switch(mode){
   case 0:break;
   case 1:max+=_image.width()+_image.height();break;
   case 2:max+=_image.width();break;
   default:max+=_image.width()-2*edge;
 }



 progressDialog pb(QObject::trUtf8("Converting picture"),QObject::trUtf8("Cancel"),0,max);
 pb.show();


 pb.addValue(1);


//Заменяем прозрачные цвета непрозрачным белым цветом
 if(_image.hasAlphaChannel()){
   for(int i=0;i<_image.width();i++)
     for(int j=0;j<_image.height();j++){
       int alpha=qAlpha(_image.pixel(i,j));
       if(alpha==0)
         _image.setPixel(i,j,0xffffffff);
     }
 }
//------------------------------------------------------------------------------


   _image=_image.convertToFormat(QImage::Format_RGB32,Qt::ThresholdDither);



 pb.addValue(1);


//Применяем фильтры для 32-битного изображения
 if(mode>2){
   _imageFilter imageFilter(edge);
   _image=imageFilter.filter(_image,mode-2,&pb);
 }


 pb.addValue(1);
 if(_image.isNull()){
   clear();
   return false;
 }



//Применяем k-mean color reduction algorithm (Lloyd's K-Means algorithm)
//[1] http://www.cescg.org/CESCG-2007/papers/Brno-Mikolov-Tomas.pdf
//[2] http://crsouza.blogspot.com/2010/10/k-means-clustering.html

//Заполняем inputVector
 QList<QRgb>inputVector;
 const int inputVectorSize=1000;//см. [1]
 const int highColorSize=16;
 if((_image.width()*_image.height())>inputVectorSize){
   for(int i=0;i<inputVectorSize;i++){
     int x=random(_image.width());
     int y=random(_image.height());
     QRgb cl=_image.pixel(x,y);
     if(clSize>highColorSize)cl=noise(cl);//[3] Добавляем к пикселю с немного шума,
     //при большом количестве нужных цветов (clSize), это улучшает подбор цветов DMC для
     //немасштабированных картинок (если картинка ранее масштабировалась, то
     //шум добавляется сам собой из-за масштабирования)
     inputVector.append(cl);
   }
 }else{
   for(int i=0;i<_image.width();i++)
     for(int j=0;j<_image.height();j++){
       QRgb cl=_image.pixel(i,j);
       if(clSize>highColorSize)cl=noise(cl);//см. [3]
       inputVector.append(cl);
     }
 }
//------------------------------------------------------------------------------

//Инициализируем centroid и colorGroup
 QList<QRgb>centroid;
 QList<QList<QRgb> >colorGroup;
 for(int i=0;i<clSize;i++){
   int k=random(inputVector.size());
   QRgb cl=inputVector.at(k);
   centroid.append(cl);
   colorGroup.append(QList<QRgb>()<<cl);
 }
//------------------------------------------------------------------------------

//Главный цикл алгоритма
 for(uchar k=0;k<iteration;k++){
   //Заполняем colorGroup близлежащими точками inputVector
   for(int i=0;i<inputVector.size();i++){
     float distance=500;//> sqrt(3*255*255)
     QRgb  inputColor=inputVector.at(i);
     int   groupNumber=0;
     for(int j=0;j<centroid.size();j++){
       float currentDistance=_dmc::distance(inputColor,centroid.at(j));
       if(currentDistance<distance){
         distance=currentDistance;
         groupNumber=j;
       }
     }
     colorGroup[groupNumber].append(inputColor);
   }
   //-----------------------------------------------------------------------------

   //Пересчитываем массив centroid, как среднее между точками попавшими в соотв. colorGroup
   centroid.clear();
   for(int i=0;i<colorGroup.size();i++){
     int groupSize=colorGroup.at(i).size();
     QRgb color=_dmc::blank();
     if(groupSize!=0){
       int r=0,g=0,b=0;
       for(int j=0;j<groupSize;j++){
         r+=qRed(colorGroup.at(i).at(j));
         g+=qGreen(colorGroup.at(i).at(j));
         b+=qBlue(colorGroup.at(i).at(j));
       }
       color=qRgb(int(r/groupSize),int(g/groupSize),int(b/groupSize));
     }else{
       int pos=random(inputVector.size());
       color=inputVector.at(pos);
     }
     centroid.append(color);
   }
   //---------------------------------------------------------------------------

   //Очищаем colorGroup
   colorGroup.clear();
   for(int i=0;i<centroid.size();i++)colorGroup.append(QList<QRgb>());
   //---------------------------------------------------------------------------
   pb.addValue(1);
 }
//------------------------------------------------------------------------------



 //Настойчивый поиск похожих цветов DMC
 for(int i=0;i<centroid.size();i++){
   QRgb clDMC=dmc.find(QColor(centroid.at(i)));//Находим максимально похожий цвет DMC
   int pos=0;
   //Если найденный цвет уже есть в палитре, ищем менее похожие цвета пока не найдём
   //новый цвет или пока не получим blank(), что означает, что хоть как-то похожих и
   //не присутствующих в палитре цветов нет
   while(colorTable.contains(clDMC)){
     clDMC=dmc.find(QColor(centroid.at(i)),++pos);
     if(clDMC==_dmc::blank())break;
   }
   //---------------------------------------------------------------------------
   if(clDMC!=_dmc::blank())colorTable.append(clDMC);
 }
 //----------------------------------------------------------------------------



 pb.addValue(1);


 for(int i=0;i<QColorDialog::customCount();i++)
   if(i<colorTable.size())QColorDialog::setCustomColor(i,colorTable.at(i));



 _image=_image.convertToFormat(QImage::Format_Indexed8,colorTable.toVector(),Qt::ThresholdDither);



 if(mode==1)
   _image=_imageFilter::median(_image,colorTable.toVector(),&pb);//Применяем медианный фильтр для 8-битного изображения

 if(mode==2)
   _image=_imageFilter::medium(_image,&pb);//Применяем усредняющий фильтр для 8-битного изображения


 pb.addValue(1);
 if((_image.isNull())||(pb.wasCanceled())){
   clear();
   return false;
 }


 pb.close();
 return true;
}

