/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef SPRITE_H
#define SPRITE_H

#include<QPoint>
#include<QColor>


class Sprite{
public:
  enum Stitch{Cross,Left,Right};
  Sprite(const QPoint &pos=QPoint(0,0),const QColor &cl=QColor(),int icon=-1,Stitch st=Cross):
    _pos(pos),
    _color(cl.rgb()),
    _iconID(0),
    _hasIcon(false),
    _stitch(st)
  {if(icon>-1)setIconID(icon);}

  void setColor(const QColor &cl){_color=cl.rgb();}
  void setIconID(uint id){_iconID=id;_hasIcon=true;}
  void setPos(const QPoint &pos){_pos=pos;}
  void setStitch(const Stitch& st){_stitch=st;}

  QPoint pos()const{return _pos;}
  QColor color()const{return QColor(_color);}
  uint   iconID()const{return _iconID;}
  Stitch stitch()const{return _stitch;}
  bool   hasIcon()const{return _hasIcon;}
private:
  //Добавляя сюда поля данных, помни о View::createSprite, там нужно инициализировать новое поле
  QPoint _pos;//Координаты вывода активной точки
  QRgb   _color;
  uint   _iconID;//Идентификатор иконки отображения цвета
  bool   _hasIcon;
  Stitch _stitch;
};

#endif // SPRITE_H
