/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "view.h"
#include "info.h"
#include "imagefilter.h"
#include "toolbar.h"



const char NO_SELECTION=-1;


View::View(QSize size,QWidget *parent):QWidget(parent){
 clear();
 setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
 setMouseTracking(true);

 _layout=new QGridLayout(this);
 _size=size;
 _icons=false;
 _grid=false;
 _spriteIconInvert=false;
 _spriteLine=false;
 _dx=0;
 _clpicCount=0;
 _backColor=Qt::transparent;
 stitch=Sprite::Cross;
 lstitch.load(Info::dataPath()+"/system/clpics/lstitch.bmp");
 rstitch.load(Info::dataPath()+"/system/clpics/rstitch.bmp");
}



QPalette View::transparency(){
 QPalette palette;
    QBrush brush(QColor(0, 0, 0, 255));
    brush.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::WindowText, brush);
    QBrush brush1(QColor(255, 255, 255, 0));
    brush1.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::Button, brush1);
    palette.setBrush(QPalette::Active, QPalette::Light, brush1);
    palette.setBrush(QPalette::Active, QPalette::Midlight, brush1);
    QBrush brush2(QColor(127, 127, 127, 0));
    brush2.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::Dark, brush2);
    QBrush brush3(QColor(170, 170, 170, 0));
    brush3.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::Mid, brush3);
    palette.setBrush(QPalette::Active, QPalette::Text, brush);
    QBrush brush4(QColor(255, 255, 255, 255));
    brush4.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::BrightText, brush4);
    palette.setBrush(QPalette::Active, QPalette::ButtonText, brush);
    palette.setBrush(QPalette::Active, QPalette::Base, brush4);
    palette.setBrush(QPalette::Active, QPalette::Window, brush1);
    palette.setBrush(QPalette::Active, QPalette::Shadow, brush1);
    QBrush brush5(QColor(10, 36, 106, 255));
    brush5.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::Highlight, brush5);
    QBrush brush6(QColor(255, 255, 255, 127));
    brush6.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::AlternateBase, brush6);
    QBrush brush7(QColor(255, 255, 220, 255));
    brush7.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::ToolTipBase, brush7);
    palette.setBrush(QPalette::Active, QPalette::ToolTipText, brush);
    palette.setBrush(QPalette::Inactive, QPalette::WindowText, brush);
    palette.setBrush(QPalette::Inactive, QPalette::Button, brush1);
    palette.setBrush(QPalette::Inactive, QPalette::Light, brush1);
    palette.setBrush(QPalette::Inactive, QPalette::Midlight, brush1);
    palette.setBrush(QPalette::Inactive, QPalette::Dark, brush2);
    palette.setBrush(QPalette::Inactive, QPalette::Mid, brush3);
    palette.setBrush(QPalette::Inactive, QPalette::Text, brush);
    palette.setBrush(QPalette::Inactive, QPalette::BrightText, brush4);
    palette.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
    palette.setBrush(QPalette::Inactive, QPalette::Base, brush4);
    palette.setBrush(QPalette::Inactive, QPalette::Window, brush1);
    palette.setBrush(QPalette::Inactive, QPalette::Shadow, brush1);
    palette.setBrush(QPalette::Inactive, QPalette::Highlight, brush1);
    palette.setBrush(QPalette::Inactive, QPalette::AlternateBase, brush6);
    palette.setBrush(QPalette::Inactive, QPalette::ToolTipBase, brush7);
    palette.setBrush(QPalette::Inactive, QPalette::ToolTipText, brush);
    palette.setBrush(QPalette::Disabled, QPalette::WindowText, brush2);
    palette.setBrush(QPalette::Disabled, QPalette::Button, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::Light, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::Midlight, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::Dark, brush2);
    palette.setBrush(QPalette::Disabled, QPalette::Mid, brush3);
    palette.setBrush(QPalette::Disabled, QPalette::Text, brush2);
    palette.setBrush(QPalette::Disabled, QPalette::BrightText, brush4);
    palette.setBrush(QPalette::Disabled, QPalette::ButtonText, brush2);
    palette.setBrush(QPalette::Disabled, QPalette::Base, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::Window, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::Shadow, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::Highlight, brush5);
    palette.setBrush(QPalette::Disabled, QPalette::AlternateBase, brush1);
    palette.setBrush(QPalette::Disabled, QPalette::ToolTipBase, brush7);
    palette.setBrush(QPalette::Disabled, QPalette::ToolTipText, brush);
 return palette;
}




//------------------------------------------------------------------------------
//---Обеспечение работы заливки области
//------------------------------------------------------------------------------

QColor View::getColor(const QPoint &pos)const{
  int i=getIndexSprite(pos);
  if((i>=0)&&(i<sprite.size()))return sprite.at(i)->color();
  return QColor();
}


void View::setColor(const QPoint &pos,const QColor &color){
  int i=getIndexSprite(pos);
  if((i>=0)&&(i<sprite.size()))sprite[i]->setColor(color);
}


void View::spriteFill(const QPoint &pos,const QColor &filledColor,const QColor &color)
{
  QColor cl=getColor(pos);
  if(cl.isValid()){
    int i=getIndexSprite(pos);
    if((i>=0)&&(i<sprite.size()))
      if((cl==filledColor)&&((cl!=color)||(sprite.at(i)->iconID()!=_iconSelected))){
        spriteFill(i);
        spriteFill(QPoint(pos.x()-int(_squarePix*_scale),pos.y()),filledColor,color);
        spriteFill(QPoint(pos.x()+int(_squarePix*_scale),pos.y()),filledColor,color);
        spriteFill(QPoint(pos.x(),pos.y()-int(_squarePix*_scale)),filledColor,color);
        spriteFill(QPoint(pos.x(),pos.y()+int(_squarePix*_scale)),filledColor,color);
      }
  }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------


void View::spriteFill(int i)
{
     history.append(i,sprite.at(i)->iconID(),sprite.at(i)->color());

     sprite.at(i)->setColor(_colorSelected);
     sprite.at(i)->setIconID(_iconSelected);
     emit updated();
}


void View::spriteFill(const QPoint &pos)
{
 int i=getIndexSprite(pos);
 if((i>=0)&&(i<sprite.size()))
   if((sprite.at(i)->color()!=_colorSelected)||(sprite.at(i)->iconID()!=_iconSelected))spriteFill(i);
}


int View::getIndexSprite(const QPoint &pos)const{
  int x=int((pos.x()/_scale-_dx)/_squarePix);
  int y=int((pos.y()/_scale-_dx)/_squarePix);
  return x*_size.height()+y;
}


int View::getIndexLine(const Line& ln)const{
  for(int i=0;i<line.size();i++)
    if(ln==*line.at(i))return i;
  return line.size();
}



void View::mouseMoveEvent(QMouseEvent *event)
{
  if(_spriteLine){
    spriteFill(event->pos());
    update();
  }

  //---Рисование линии
  if((linePreview.mode==LineMode::Add)&&(!linePreview.line.p1().isNull())){
    linePreview.line.setP2(linePoint(event->pos()));
    update();
  }
  //------------------------------------------------------------------------------

  //---Подсветка линии для удаления
  if(linePreview.mode==LineMode::Del){
    for(int i=0;i<line.size();i++)
      if(line.at(i)->hasPoint(event->pos()/_scale)){
        _lineSelected=i;
        linePreview.line.setPoints(line.at(i)->p1(),line.at(i)->p2());
        linePreview.line.setColor(invertColor(line.at(i)->color()));
        linePreview.line.setWidth(5);
        update();
        break;
      }
  }

  emit mousePos(event->pos());
}



void View::mouseReleaseEvent(QMouseEvent* event)
{
  if(_spriteLine)_spriteLine=false;

  //---Рисование линии
  if((linePreview.mode==LineMode::Add)&&(!linePreview.line.isNull())){
      linePreview.line.setP2(linePoint(event->pos()));

      int id=getIndexLine(linePreview.line);
      if(id==line.size()){
      //Если линия не совпадает ни с одной линией из line
        try{
          line.append(new Line(linePreview.line));
          history.append(new LineItem(linePreview.line,&line));
        }catch(const std::bad_alloc&){
          Info::messageSplash(Info::msgOutOfMemory);
        }
      }
      emit updated();
      update();
      linePreview.line.clear();
  }
  //------------------------------------------------------------------------------
}



void View::mousePressEvent(QMouseEvent *event)
{
  if(event->button()==Qt::LeftButton){
    int toolID=ToolBar::instance()->buttonGroup()->checkedId();
    if(_colorSelected!=Qt::transparent){
      if(_colorSelected!=_backColor){
        //---Рисование линии
        if((linePreview.mode==LineMode::Add)&&(linePreview.line.isNull())){
          linePreview.line.setP1(linePoint(event->pos()));
          linePreview.line.setColor(_colorSelected);
        }
      //------------------------------------------------------------------------------
      }

      //---Заливка
      if(toolID==ToolBar::colorFill){
        try{
          history.append(new SpriteItem(&sprite));
        }catch(const std::bad_alloc&){
          Info::messageSplash(Info::msgOutOfMemory);
        }
        spriteFill(event->pos(),getColor(event->pos()),_colorSelected);
        update();
      }
      //------------------------------------------------------------------------------

      //if(_iconSelected!=NO_SELECTION){
        //---Карандаш (рисование линии спрайтов)
        if(toolID==ToolBar::penDraw){
          try{
            history.append(new SpriteItem(&sprite));
          }catch(const std::bad_alloc&){
            Info::messageSplash(Info::msgOutOfMemory);
          }
          spriteFill(event->pos());
          _spriteLine=true;
          update();
        }
        //------------------------------------------------------------------------------
      //}
    }

    //---Удаление линии
    if((linePreview.mode==LineMode::Del)&&(_lineSelected!=NO_SELECTION)){
      try{
        history.append(new LineItem(*line.at(_lineSelected),&line));
      }catch(const std::bad_alloc&){
        //Exception handle
        Info::messageSplash(Info::msgOutOfMemory);
      }
      delete line.takeAt(_lineSelected);
      linePreview.line.clear();
      _lineSelected=NO_SELECTION;
      emit updated();
      update();
    }
    //--------------------------------------------------------------------------
    //---Выбор цвета
    if(toolID==ToolBar::colorSelect){
       int i=getIndexSprite(event->pos());
       if((i>=0)&&(i<sprite.size())){
           QPixmap px(20,20);
           _colorSelected=sprite.at(i)->color();
           _iconSelected=sprite.at(i)->iconID();
           px.fill(_colorSelected);
           emit colorSelected(px);
       }
    }
    //------------------------------------------------------------------------------
    if(toolID==ToolBar::stitchType){
      int i=getIndexSprite(event->pos());
      if((i>=0)&&(i<sprite.size())){
        if(sprite.at(i)->stitch()!=stitch){
          try{
            history.append(new SpriteItem(&sprite));
            history.append(i,sprite.at(i)->iconID(),sprite.at(i)->color(),sprite.at(i)->stitch());
          }catch(const std::bad_alloc&){
            //Exception handle
            Info::messageSplash(Info::msgOutOfMemory);
          }
          sprite.at(i)->setStitch(stitch);
        }
      }
      update();
    }
  }
}


void View::setSquarePix(int pix){
  _squarePix=pix;
}


void View::clear(){
  foreach(Sprite* pix,sprite)free(pix);
  sprite.clear();
  while(!line.isEmpty())delete line.takeFirst();
  line.clear();

  _scale=1;
  _iconSelected=NO_SELECTION;
  _lineSelected=NO_SELECTION;
  linePreview.line.clear();
  setColorSelected(Qt::transparent);
  history.clear();
}



void View::paintEvent(QPaintEvent *){
 QPainter painter(this);

 painter.scale(_scale,_scale);

 painter.setPen(_pen);
 painter.setFont(_font);

 QSize size(_squarePix,_squarePix);
 setStitchImages(size);

 //QTime t;
 //t.start();

 QColor cashedColor(Qt::black);
 QPixmap cashedPixmap(size);
 cashedPixmap.fill(cashedColor);

 uint cashedIconID=INT_MAX;
 QImage cashedImage;//=imageList.at(cashedIconID);

 foreach(Sprite* pix,sprite){

   if(cashedColor!=pix->color()){
     cashedColor=pix->color();
     cashedPixmap.fill(cashedColor);
   }
   painter.drawPixmap(pix->pos(),cashedPixmap);//drawPixmap быстрее, чем drawRect

   if(_icons){
       if(pix->stitch()==Sprite::Left)painter.drawImage(pix->pos(),lst);
       if(pix->stitch()==Sprite::Right)painter.drawImage(pix->pos(),rst);

       if((pix->hasIcon())&&(pix->iconID()<_clpicCount)){
         if(cashedIconID!=pix->iconID()){
           cashedIconID=pix->iconID();
           cashedImage=imageList.at(cashedIconID);
           if(_spriteIconInvert)cashedImage.invertPixels();
         }
         painter.drawImage(pix->pos()+_dl,cashedImage);
       }else{
         if(pix->iconID()!=INT_MAX)painter.drawText(QRect(pix->pos()+_dl,QSize(_squarePix-2*_dl.x(),_squarePix-2*_dl.y())),Qt::AlignCenter,Info::colorSign(pix->iconID()));
       }
   }
 }

 //qDebug("Time elapsed: %d ms", t.elapsed());

 showLines(&painter);
 int toolID=ToolBar::instance()->buttonGroup()->checkedId();
 if(toolID==ToolBar::colorShow)showColor(&painter);
 if(_grid)showGrid(&painter);


 //---Рисование временной линии
     if(linePreview.mode==LineMode::Add){
       painter.setPen(linePreview.line.pen());
       painter.drawLine(linePreview.line);
     }

     if((linePreview.mode==LineMode::Del)&&(!linePreview.line.isNull())){
       painter.setPen(QPen(QBrush(linePreview.line.color(),Qt::Dense3Pattern),linePreview.line.width(),Qt::DotLine));
       painter.drawLine(linePreview.line);
     }

 //------------------------------------------------------------------------------


}



QImage View::toImage(bool fastPreview){

  float scale=_imageFilter::scaleFit(_size*_squarePix,12);//Этот коэфф. нужен для масштабирования, т.к. QImage не работает с большими картинками

  QImage img((_size*_squarePix+2*QSize(_dx,_dx))*scale,QImage::Format_RGB16);

  img.fill(Qt::white);//Надо обязательно вызывать перед рисованием на painter(&img)
  if(img.isNull())return img;
  
  QPainter painter(&img);
  painter.scale(scale,scale);
  painter.fillRect(img.rect(),Qt::white);//Если этого не сделать картинка будет чёрной независимо от вызова img.fill(Qt::white)
  QSize size(_squarePix,_squarePix);
  QPixmap pixSprite(size);

  setStitchImages(size);

  foreach(Sprite* pix,sprite){
    pixSprite.fill(pix->color());
    painter.drawPixmap(pix->pos(),pixSprite);

    if(!fastPreview){
        if(pix->stitch()==Sprite::Left)painter.drawImage(pix->pos(),lst);
        if(pix->stitch()==Sprite::Right)painter.drawImage(pix->pos(),rst);

        if(pix->iconID()<_clpicCount){
          if(_spriteIconInvert){
            QImage img=imageList.at(pix->iconID());
            img.invertPixels();
            painter.drawImage(pix->pos()+_dl,img);
          }else painter.drawImage(pix->pos()+_dl,imageList.at(pix->iconID()));
        }else
          if(pix->iconID()!=INT_MAX)painter.drawText(QRect(pix->pos()+_dl,QSize(_squarePix-2*_dl.x(),_squarePix-2*_dl.y())),Qt::AlignCenter,Info::colorSign(pix->iconID()));
    }
  }

  showLines(&painter);
  if(!fastPreview)showGrid(&painter);
  
  return img;
}



void View::setStitchImages(const QSize& size){
    lst=lstitch.scaled(size);
    lst.setAlphaChannel(lst.createMaskFromColor(0xFFFFFFFF,Qt::MaskOutColor));
    rst=rstitch.scaled(size);
    rst.setAlphaChannel(rst.createMaskFromColor(0xFFFFFFFF,Qt::MaskOutColor));
    if(_spriteIconInvert){
      lst.invertPixels();
      rst.invertPixels();
    }
}


QImage View::toImage(QRect rect){
  //Функция используется для печати на принтере
  //Размерность rect - спрайты!
  QRect r(QPoint(rect.topLeft()*_squarePix),QSize(rect.size()*_squarePix));

  QImage img(r.size()*_scale,QImage::Format_ARGB32_Premultiplied);

  img.fill(Qt::white);
  if(img.isNull())return img;

  QPainter painter(&img);
  painter.scale(_scale,_scale);



  QSize size(_squarePix,_squarePix);
  QPixmap pixSprite(size);

  setStitchImages(size);


  int x1=r.bottomRight().x();
  int y1=r.bottomRight().y();
  int x2=r.x();
  int y2=r.y();

  foreach(Sprite* pix,sprite){
    QRect spr(pix->pos(),QSize(_squarePix,_squarePix));
    if(r.contains(spr)){

      if(spr.x()<x1)x1=spr.x();
      if(spr.y()<y1)y1=spr.y();
      if(x2<spr.bottomRight().x())x2=spr.bottomRight().x();
      if(y2<spr.bottomRight().y())y2=spr.bottomRight().y();


      pixSprite.fill(pix->color());
      painter.drawPixmap(pix->pos()-r.topLeft(),pixSprite);

      if(pix->stitch()==Sprite::Left)painter.drawImage(pix->pos()-r.topLeft(),lst);
      if(pix->stitch()==Sprite::Right)painter.drawImage(pix->pos()-r.topLeft(),rst);


        if(pix->iconID()<_clpicCount){
          if(_spriteIconInvert){
            QImage img=imageList.at(pix->iconID());
            img.invertPixels();
            painter.drawImage(pix->pos()-r.topLeft()+_dl,img);
          }else painter.drawImage(pix->pos()-r.topLeft()+_dl,imageList.at(pix->iconID()));
        }else
          if(pix->iconID()!=INT_MAX)painter.drawText(QRect(pix->pos()-r.topLeft()+_dl,QSize(_squarePix-2*_dl.x(),_squarePix-2*_dl.y())),Qt::AlignCenter,Info::colorSign(pix->iconID()));
    }
  }
  showLines(&painter,-r.topLeft());
  showGrid(&painter,QRect(QPoint(x1,y1)-r.topLeft(),QPoint(x2,y2)-r.topLeft()),r.x(),r.y());

  return img;
}




void View::showGrid(QPainter *surf,QRect r,int counterX,int counterY)const
{
  if(r.isNull())r=QRect(QPoint(_dx,_dx),_size*_squarePix);

  QBrush brush(Qt::SolidPattern);
  QPen pen(brush,_linePix);
  QPen penBold(brush,_linePix+2);

  QSize _background(r.size());

  if(counterX!=0)counterX-=_squarePix;
  if(counterY!=0)counterY-=_squarePix;



  for(int i=r.y();i<=_background.height()+r.y();i=i+_squarePix){
    surf->setPen(pen);
    int dl=0;
    int count=counterY+i-r.y();
    if(count%(10*_squarePix)==0){
      surf->setPen(penBold);
      dl=_dx;
      if(((i-r.y())!=0)&&(counterX==0))
        surf->drawText(QRect(1,i-dl,dl-1,dl-1),Qt::AlignCenter,QString::number(int(count/_squarePix)));
    }
    surf->drawLine(r.x()-dl,i,_background.width()+r.x()+dl,i);
  }

  for(int i=r.x();i<=_background.width()+r.x();i=i+_squarePix){
    surf->setPen(pen);
    int dl=0;
    int count=counterX+i-r.x();
    if(count%(10*_squarePix)==0){
      surf->setPen(penBold);
      dl=_dx;
      if(((i-r.x())!=0)&&(counterY==0))
        surf->drawText(QRect(i-dl,1,dl-1,dl-1),Qt::AlignCenter,QString::number(int(count/_squarePix)));
    }
    surf->drawLine(i,r.y()-dl,i,_background.height()+r.y()+dl);
  }

}


QColor View::invertColor(const QColor& cl)const
{
  int r=255-cl.red();
  int b=255-cl.blue();
  int g=255-cl.green();
  return QColor(r,g,b);
}


void View::showColor(QPainter *surf)const
{
  QColor cl=invertColor(_colorSelected);

  QBrush brush(cl,Qt::Dense4Pattern);

  surf->setBrush(brush);
  surf->setPen(Qt::NoPen);
  QSize size(_squarePix,_squarePix);

  foreach(Sprite* pix,sprite)
    if(_colorSelected==pix->color()){
      surf->drawRect(QRect(pix->pos(),size));
    }
}


void View::showLines(QPainter *surf, const QPoint &dx)const
{
  foreach(Line* ln,line){
    if(surf->pen()!=ln->pen())surf->setPen(ln->pen());
    Line line(*ln);
    line.translate(dx);
    surf->drawLine(line);
  }
}


void View::colorUpdate(const QColor &clOld,const QColor &clNew){
  foreach(Sprite* sp,sprite)
    if(sp->color()==clOld)sp->setColor(clNew);
  foreach(Line* ln,line)
    if(ln->color()==clOld)ln->setColor(clNew);
}




void View::iconUpdate(const QColor &cl,int num){
  for(int i=0;i<sprite.size();i++)
    if(sprite.at(i)->color()==cl)
      sprite.at(i)->setIconID(num);
}


void View::setBackground(const QColor &cl){
  foreach(Sprite* pix,sprite)
    //Цвет = _bkColor и iconID = INT_MAX определяют цвет фона.
    if((pix->color()==_backColor)&&(pix->iconID()==INT_MAX))pix->setColor(cl);
  _backColor=cl;
}



int View::colorCount(const QColor &cl)const{
  int result=0;
  foreach(Sprite* pix,sprite)
    if(cl==pix->color())result++;
  foreach(Line* ln,line)
    if(cl==ln->color())result++;
  return result;
}



int View::colorCount(QRgb rgbcl)const{
  QColor cl;
  cl.setRgba(rgbcl);
  return colorCount(cl);
}



void View::setParameters(int squarePix,int imgW,int imgH,int dX,int linePix,int dl){
  setSquarePix(squarePix);
  setDX(dX);
  _size=QSize(imgW,imgH);
  setLinePix(linePix);
  setDL(QPoint(dl,dl));
}


void View::createLine(const QPoint &p1,const QPoint &p2,const QColor &cl,int w){
  try{
    line.append(new Line(p1,p2,w,cl));
  }catch(const std::bad_alloc&){
    //Exception handle
    Info::messageSplash(Info::msgOutOfMemory);
  }
}


bool View::createSprite(const QPoint &point,const QColor &cl,int id){
  try{
    sprite.append(new Sprite(point,cl,id));
  }catch(const std::bad_alloc&){
    return false;
  }
  return true;
}


ColorItem* View::colorItem(int row,const QColor& cl){
 return new ColorItem(row,cl,&sprite,&line);
}


IconItem* View::iconItem(int row,const QColor& cl){
 return new IconItem(row,cl,&sprite);
}


Sprite* View::getSprite(int i)const{
  if(i<sprite.size())return sprite[i];
  return NULL;
}


Line* View::getLine(int i)const{
  if(i<line.size())return line[i];
  return NULL;
}


void View::setColorSelected(const QColor &cl){
  _colorSelected=cl;
  QPixmap px(20,20);
  px.fill(_colorSelected);
  emit colorSelected(px);
}


QPoint View::linePoint(const QPoint &pos)const
{
  int dl=int(_squarePix/2);

  int x=int((pos.x()/_scale)/_squarePix)*_squarePix+dl;
  int max=_dx+_size.width()*_squarePix;
  if(x>max)x=max;
  if(x<_dx)x=_dx;

  int y=int((pos.y()/_scale)/_squarePix)*_squarePix+dl;
  max=_dx+_size.height()*_squarePix;
  if(y>max)y=max;
  if(y<_dx)y=_dx;

  return QPoint(x,y);
}

