/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "viewer.h"
#include "info.h"

viewer::viewer(_schemeData* schemed,QWidget *parent):QWidget(parent),
  schemeData(schemed)
{
 ui.setupUi(this);
 ui.schemeBtn->setVisible(false);

 setWindowIcon(QIcon(Info::dataPath()+"/system/image/crosti.png"));

 imageScene=new QGraphicsScene;
 ui.imageView->setScene(imageScene);

 if((schemeData!=NULL)&&(!schemeData->isNull())){
     setImage(schemeData->image());
     ui.schemeBtn->setVisible(true);
 }

 ui.schemeBtn->setIcon(QIcon(Info::dataPath()+"/system/image/2.png"));
 ui.scaleResetBtn->setIcon(QIcon(Info::dataPath()+"/system/image/zoomreset.png"));

 connect(ui.scaleSB,SIGNAL(valueChanged(int)),SLOT(setScale(int)));
 connect(ui.scaleResetBtn,SIGNAL(clicked()),SLOT(scaleResetBtnPress()));
 connect(ui.schemeBtn,SIGNAL(clicked()),SLOT(schemeBtnPress()));

 setAttribute(Qt::WA_DeleteOnClose);
}


viewer::~viewer()
{
  ui.imageView->~QGraphicsView();
  ui.schemeBtn->~QToolButton();
  ui.label_4->~QLabel();
  ui.scaleSlider->~QSlider();
  ui.scaleSB->~QSpinBox();
  ui.horizontalSpacer_2->~QSpacerItem();
  ui.scaleResetBtn->~QToolButton();
  ui.horizontalSpacer->~QSpacerItem();
  ui.horizontalLayout_2->~QHBoxLayout();
  ui.gbImage->~QGroupBox();
  ui.verticalLayout->~QVBoxLayout();

  imageScene->~QGraphicsScene();
  _image=QImage();
  schemeData->~_schemeData();
  schemeData=NULL;
}


void viewer::setImage(const QImage &img)
{
 _image=img;

 imageScene->clear();
 imageScene->setSceneRect(img.rect());
 imageScene->addPixmap(QPixmap::fromImage(_image));
 ui.scaleSB->setValue(100);
 emit imageChanged(_image);
}


void viewer::setScale(int factor)
{
 ui.imageView->resetTransform();
 qreal scale=qreal(factor)/100;
 ui.imageView->scale(scale,scale);
}


void viewer::schemeBtnPress()
{
  emit makeScheme(schemeData);
}


void viewer::scaleResetBtnPress()
{
 ui.scaleSB->setValue(100);
}
