/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef WIZARD_H
#define WIZARD_H


#include "schemedata.h"
#include "viewer.h"

#include <QtGui>



class crostiWizard:public QWizard
{
     Q_OBJECT
 public:

     QList<_colorCount>colorCount;
     QList<QRgb>colorTable;

     crostiWizard(QWidget *parent = 0);
     ~crostiWizard(){
       clear();
     }

     void setImage(const QImage &img){_image=img;}
     QImage image()const{return _image;}
     void setImageOriginal(const QImage &img){_imageOriginal=img;}
     QImage imageOriginal()const{return _imageOriginal;}

     void accept();
     void reject();

 protected:
   void closeEvent(QCloseEvent *e){
     clear();
     saveRegistry();
     e->accept();
   }

 private:
     QImage _image,
            _imageOriginal;

     void clear(){
       colorTable.clear();
       colorCount.clear();
       _image=QImage();
       _imageOriginal=QImage();
     }

     void saveRegistry()const{
       QSettings rsettings(QApplication::organizationName(),QApplication::applicationName());
       rsettings.setValue("wizard",field("wizard").toBool());
     }

};




class introPage:public QWizardPage
{
     Q_OBJECT
public:
     introPage(QWidget *parent = 0);
protected:
     void initializePage();

private:

     QLabel *label;
     QCheckBox *chbox;

     bool isStartup()const{
       QSettings rsettings(QApplication::organizationName(),QApplication::applicationName());
       return rsettings.value("wizard").value<bool>();
     }
};



class conclusionPage : public QWizardPage
 {
     Q_OBJECT
 public:
     conclusionPage(QWidget *parent = 0);
 private:
     QLabel *label;
 };



class openImagePage : public QWizardPage
{
     Q_OBJECT
 public:
     openImagePage(QWidget *parent=0);
 protected:
    void initializePage();
 private slots:
     void openDialog();
 private:
     viewer *imageViewer;
     QPushButton *openImageAct;
};



class convertImagePage : public QWizardPage
{
     Q_OBJECT
 public:
     convertImagePage(crostiWizard *cwp,QWidget *parent = 0);
     bool isComplete()const{return _complete;}

 private slots:
    void convertImage();
 protected:
     void initializePage();

 private:

     crostiWizard *_parent;
     viewer *imageViewer;
     QSpinBox *widthSB,*heightSB,*colorSB;
     QPushButton *convertImageAct;
     _dmc dmc;
     bool _complete;


     void setComplete(const bool complete){
       _complete=complete;
       emit completeChanged();
     }
};




#endif // WIZARD_H
